/*
 * Copyright (C) 1999 Matt Newman <matt@sensus.org>
 *
 * $Header: /cvsroot/metakit/mk/tcl/kit/vfs.c,v 1.1 2000/11/08 22:47:57 jcw Exp $
 */

#include <tcl.h>

DLLEXPORT int Vfs_Init	_ANSI_ARGS_((Tcl_Interp*));
static int NormObjCmd _ANSI_ARGS_((ClientData clientData,
			    Tcl_Interp* interp, int objc, Tcl_Obj*CONST objv[]));

static int
NormObjCmd(dummy, interp, objc, objv)
    ClientData dummy;
    Tcl_Interp *interp;
    int		objc;
    Tcl_Obj	*CONST objv[];
{
    Tcl_DString path;
    char *fileName;
    char **sp = NULL, *np[BUFSIZ];
    int splen = 0, nplen, i;

    if (objc != 2) {
	Tcl_WrongNumArgs(interp, 1, objv, "filename");
	return TCL_ERROR;
    }
    Tcl_DStringInit(&path);

    fileName = Tcl_GetStringFromObj(objv[1], (int *) NULL);

    if (Tcl_GetPathType(fileName) == TCL_PATH_RELATIVE) {
	char *cwd = Tcl_GetVar(interp, "::vfs::cwd", TCL_GLOBAL_ONLY);

	if (cwd) {
	    char *pair[2];

	    pair[0] = cwd;
	    pair[1] = fileName;
	    Tcl_JoinPath(2, pair, &path);
	}
	fileName = Tcl_DStringValue(&path);
    }
    Tcl_SplitPath(fileName, &splen, &sp);

    nplen = 0;
    for (i = 0;i < splen;i++) {
	if (strcmp(sp[i], ".") == 0)
	    continue;

	if (strcmp(sp[i], "..") == 0) {
	    if (nplen > 1) nplen--;
	} else {
	    np[nplen++] = sp[i];
	}
    }
    if (nplen > 0) {
	Tcl_DString ds;

	Tcl_DStringInit(&ds);

	Tcl_JoinPath(nplen, np, &ds);

	Tcl_DStringResult(interp, &ds);
	Tcl_DStringFree(&ds);
    } else {
	Tcl_SetResult( interp, "", TCL_STATIC);
    }
    Tcl_Free( (char*) sp);
    Tcl_DStringFree(&path);
    return TCL_OK;
}

int
Vfs_Init(interp)
    Tcl_Interp* interp;
{
#if TCL_MAJOR_VERSION >= 8 && TCL_MINOR_VERSION >= 2
    if (!Tcl_InitStubs(interp, "8.2", 0)) {
        return TCL_ERROR;
    }
#endif
    Tcl_CreateObjCommand(interp, "::vfs::normalize", NormObjCmd,
		  (ClientData) NULL, (Tcl_CmdDeleteProc *) NULL);

    return Tcl_PkgProvide(interp, "vfs.builtin", "0.1");
}
